/*
 * devmem2.c: Simple program to read/write from/to any location in memory.
 *
 *  Copyright (C) 2000, Jan-Derk Bakker (J.D.Bakker@its.tudelft.nl)
 *
 *
 * This software has been developed for the LART computing board
 * (http://www.lart.tudelft.nl/). The development has been sponsored by
 * the Mobile MultiMedia Communications (http://www.mmc.tudelft.nl/)
 * and Ubiquitous Communications (http://www.ubicom.tudelft.nl/)
 * projects.
 *
 * The author can be reached at:
 *
 *  Jan-Derk Bakker
 *  Information and Communication Theory Group
 *  Faculty of Information Technology and Systems
 *  Delft University of Technology
 *  P.O. Box 5031
 *  2600 GA Delft
 *  The Netherlands
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/*
 *  pcie_mwr.c: Simple program for writing to 32 bit locations in PCI memory space.
 *
 *  Copyright (C) 2015, Aleksandar Jovanovic
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * The author can be reached at:
 *
 *  Aleksandar Jovanovic (aleksandar.jovanovic@xlinix.com)
 *  IPDS
 *  Xilinx
 *  Citywest Business Campus
 *  Dublin 24
 *  Ireland
 *
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>  //dku added so can use uint8_t, uint16_t, uint32_t
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <fcntl.h>
#include <ctype.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/mman.h>
  
#define FATAL do { fprintf(stderr, "Error at line %d, file %s (%d) [%s]\n", \
  __LINE__, __FILE__, errno, strerror(errno)); exit(1); } while(0)
 
#define MAP_SIZE 4096UL
#define MAP_MASK (MAP_SIZE - 1)

int main(int argc, char **argv) {
  int fd;
  void *map_base, *virt_addr; 
  //unsigned long read_result, writeval;
  uint32_t read_result, writeval, readCount, i; //dku, ajo
  off_t target;
  int access_type = 'w';
  
  target = strtoul(argv[1], 0, 0);

  if((fd = open("/dev/mem", O_RDWR | O_SYNC)) == -1) FATAL;
  
  /* Map one page */
  map_base = mmap(0, MAP_SIZE, PROT_READ | PROT_WRITE, MAP_SHARED, fd, target & ~MAP_MASK);
  if(map_base == (void *) -1) FATAL;
  
  virt_addr = map_base + (target & MAP_MASK);

  if(argc == 3) {
    writeval = strtoul(argv[2], 0, 0);
    *((uint32_t *) virt_addr) = writeval;       //dku
  } else {
    fprintf(stderr, "\nUsage:\t%s { address } { data }\n"
      "\taddress : memory address to act upon\n"
      "\ttype    : access operation type : [w]rite, [r]ead\n"
      "\tdata    : data to be written\n\n",
      argv[0]);
    exit(1);
  }
  
  if(munmap(map_base, MAP_SIZE) == -1) FATAL;
    close(fd);
    return 0;
}

